Class: Git::Si::SvnInterface

Inherits:
Thor
  • Object
show all
Includes:
Actions, Util, Pager, Thor::Actions
Defined in:
lib/git/si.rb

Instance Method Summary collapse

Methods included from Actions

#do_add_action, #do_commit_action, #do_diff_action, #do_fetch_action, #do_init_action, #do_pull_action, #do_readd_action, #do_rebase_action, #do_status_action

Methods included from Util

#add_all_svn_files, #add_files_after_svn_update, #add_files_to_git, #are_there_git_changes?, #batch_add_files_to_git, #configure, #create_git_repository, #create_gitignore, #create_mirror_branch, #debug, #delete_committed_branch, #delete_files_after_svn_update, #did_last_command_succeed?, #do_revisions_differ, #error_message, #get_command_output, #get_git_si_revision, #get_local_branch, #get_mirror_branch, #get_svn_revision, #get_svn_root, #in_svn_root, #is_file_in_git?, #notice_message, #on_local_branch, #on_mirror_branch, #print_colordiff, #revert_files_to_svn_update, #run_command, #stash_local_changes, #success_message, #unstash_local_changes, #using_stderr

Instance Method Details

#add(*args) ⇒ Object



76
77
78
79
# File 'lib/git/si.rb', line 76

def add(*args)
  configure
  do_add_action( args )
end

#commitObject



111
112
113
114
# File 'lib/git/si.rb', line 111

def commit
  configure
  do_commit_action
end

#diff(*args) ⇒ Object



67
68
69
70
# File 'lib/git/si.rb', line 67

def diff(*args)
  configure
  do_diff_action( args )
end

#fetchObject



85
86
87
88
# File 'lib/git/si.rb', line 85

def fetch
  configure
  do_fetch_action
end

#initObject



129
130
131
132
# File 'lib/git/si.rb', line 129

def init
  configure
  do_init_action
end

#pullObject



103
104
105
# File 'lib/git/si.rb', line 103

def pull
  do_pull_action
end

#readdObject



120
121
122
123
# File 'lib/git/si.rb', line 120

def readd
  configure
  do_readd_action
end

#rebaseObject



94
95
96
97
# File 'lib/git/si.rb', line 94

def rebase
  configure
  do_rebase_action
end

#status(*args) ⇒ Object



58
59
60
61
# File 'lib/git/si.rb', line 58

def status(*args)
  configure
  do_status_action( args )
end

#usageObject



40
41
42
43
44
45
46
47
48
49
50
51
52
# File 'lib/git/si.rb', line 40

def usage
  say "#{Git::Si::Version.version_string}

Git Svn Interface: a simple git extention to use git locally with a remote svn
repository. It's like a simple version of git-svn just for using local
branching. It does not keep track of the full history of the svn repository.

Start with the init command to set up the mirror branch and from there you can
use the commands below.

"
  help
end

#versionObject



32
33
34
# File 'lib/git/si.rb', line 32

def version
  say Git::Si::Version.version_string
end